// bumpmapping.wdl

material bumpmapping_version1 { 
	flags=tangent; 
	
	effect
	" 			
		matrix matWorldViewProj;
		matrix matWorld;
		
		texture entSkin3;			// Shadow Map
		texture entSkin1;			// Color Map
		texture entSkin2;			// Bump Map
		vector  vecLight;
						
		technique dot3map
		{
			pass p0
			{
				Texture[0] = <entSkin2>;
				Texture[1] = <entSkin1>;
				Texture[2] = <entSkin3>;
				TextureFactor = 0xFFFFFFFF;
				
				COLOROP[0] = DotProduct3;
				COLORARG1[0] = Texture;
				COLORARG2[0] = TFactor;
				TexCoordIndex[0] = 1;
				
				COLOROP[1] = Modulate;
				COLORARG1[1] = Texture;
				COLORARG2[1] = Current;
				TexCoordIndex[1] = 0;
				
				magFilter[2]=Linear;
				minFilter[2]=Linear;
				mipFilter[2]=Linear;
				COLOROP[2] = Modulate;
				COLORARG1[2] = Texture;
				COLORARG2[2] = Current;
				TexCoordIndex[2] = 1;
			}
		}
	"; 
}

material bumpmapping_version2 {
	
	effect
	"
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		// Define your needed values
		float4x4 matWorldViewProj;			
		float4x4 matWorld;
		float4 	vecLight;
		float4	vecSunDir;
		float4 	vecViewDir;
		float4   vecViewPos;
			
		// Define your textures
		texture entSkin1;					
		texture entSkin2;					
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		// Texture settings

		sampler ColorMapSampler = sampler_state			
		{
			Texture = <entSkin1>;
			MipFilter = Linear;
			MinFilter = Linear;
			MagFilter = Linear;
			AddressU = Clamp;	
			AddressV = Clamp;
		};

		sampler BumpMapSampler = sampler_state			
		{
			Texture = <entSkin2>;
			MipFilter = Linear;
			MinFilter = Linear;
			MagFilter = Linear;
			AddressU = Clamp;	
			AddressV = Clamp;
		};

		
		// Define the output of your vertexshader
		struct VS_OUT
		{
			float4 Pos	 : POSITION;
			float2 Tex	 : TEXCOORD0;
			float3 Light : TEXCOORD1;
			float3 View  : TEXCOORD2;
		};

		struct VS_IN
		{
			float4 Pos		: POSITION;
			float2 Tex		: TEXCOORD;
			float3 Normal	: NORMAL;
			float3 Tangent	: TANGENT;
		};

		//////////////////////////////////////////////////////////////////////////////////////////////////////
		// vertexshader
		
		VS_OUT vs( VS_IN In )
		{
			VS_OUT Out = ( VS_OUT ) 0;							// Declare your output 
			
			Out.Pos = mul( In.Pos, matWorldViewProj );	// Transform the ouput to the view
			
			float3x3 worldToTangentSpace;
			
			worldToTangentSpace[0]=mul(In.Tangent, matWorld);
			worldToTangentSpace[1]=mul(cross(In.Tangent,In.Normal), matWorld);
			worldToTangentSpace[2]=mul(In.Normal, matWorld);
			
			Out.Tex=In.Tex;
			
			float4 PosWorld = mul(In.Pos, matWorld);
			
			Out.Light = mul(worldToTangentSpace, vecLight);
			// In the Original Shader: Out.Light = mul(worldToTangentSpace, vecLightDir);
			
			Out.View = mul(worldToTangentSpace, vecViewPos - PosWorld);
			// In the Original Shader: Out.View = mul(worldToTangentSpace, vecEye - PosWorld);
			
			return Out;
		}


		float4 ps(
			float2 Tex : TEXCOORD0, 
			float3 Light: TEXCOORD1, 
			float3 View : TEXCOORD2 
		) : COLOR
		{
			float4 Color = tex2D(ColorMapSampler, Tex);
			float3 Normal = (2*(tex2D(BumpMapSampler,Tex)))-1.0;
			
			float3 LightDir = normalize(Light);
			float3 ViewDir = normalize(View);
			
			float Diffuse=saturate(dot(Normal,LightDir));
			
			float3 Reflect = normalize(2*Diffuse * Normal - LightDir);
			
			float Specular = pow(saturate(dot(Reflect,ViewDir)),3);
			
			return 0.2 * Color + Color * Diffuse + Specular;
		}
	
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		// 
		technique mytechnique
		{
			// Define your passes
			pass p0
			{
				VertexShader = compile vs_2_0 vs();
				PixelShader = compile ps_2_0 ps();
			}
		}
	";
}


ACTION bumpmap1 {
	my.material = bumpmapping_version1;
}

ACTION bumpmap2 {
	my.material = bumpmapping_version2;
}



Function Main() {	
	video_switch(8,32,1);		// switch window to full screen 1024x768 true color

	fps_max=60;						// max framerate 60
	fps_min=20;						// min_framerate 20

	level_load("bumpmapping.wmb");
}